{Graficky unit}
{R. Novotny}

{$DEFINE WIN32}

unit grafika3D;

interface

{$IFDEF WIN32}
uses graphics;
{$ELSE}
uses dos;
{$ENDIF}

type TPoint3D = record
  x:real;
  y:real;
  z:real;
end;

type TPointH3D = record
  x:real;
  y:real;
  z:real;
  h:real;
end;

type TMatrix = array[1..3, 1..3] of real;
type TPointMatrix = array[1..3] of real;
type TShape = array[1..30] of TPointH;

const xlatematrix:TMatrix = ((1,0,0),(0,1,0),(MAXINT, MAXINT, 1));
const scalematrix:TMatrix = ((MAXINT,0,0),(0,MAXINT,0),(0, 0, 1));
const rotatematrix:TMatrix = ((MAXINT,MAXINT,0),(MAXINT,MAXINT,0),(0, 0, 1));

{$IFDEF WIN32}
var mem:TCanvas;
{$ELSE}
var mem:array [0..$7FFF] of byte absolute $A000:0000;
{$ENDIF}

{$IFDEF WIN32}
procedure init(canvas:TCanvas);
{$ELSE}
procedure init(mode:byte);
{$ENDIF}
procedure usecka_b(x1, y1, x2, y2: integer; farba: byte);
procedure bod_pc(x,y:integer; farba:byte);
function farbabodu_pc(x,y:integer):byte;
procedure PointToPointH(pt:TPoint; var pth:TPointH);
procedure PointHToPoint(pth:TPointH; var pt:TPoint);
procedure PointMatrixTimesMatrix(pt:TPointMatrix; m:TMatrix; var result:TPointMatrix);
procedure PointHToPointMatrix(pth:TPointH; var ptm:TPointMatrix);
procedure PointMatrixToPointH(ptm:TPointMatrix; var pth:TPointH);
procedure multiplyMatrices(m,n: TMatrix; var p: TMatrix);
procedure getTranslationMatrix(dx, dy:integer; var m:TMatrix);
procedure getRotationMatrix(angle:real; var m:TMatrix);
procedure translatePointH(pt:TPointH; dx,dy:integer; var xpth:TPointH);
procedure scalePointH(pt:TPointH; xscale, yscale:real; var xpth:TPointH);
procedure rotatePointH(pt:TPointH; angle:real; var xpth:TPointH);
procedure rotatePointHEx(pt:TPointH; angle:real; xoffset, yoffset:integer; var xpth:TPointH);

implementation

{$IFDEF WIN32}
procedure init(canvas:TCanvas);
begin
  mem:=canvas;
end;
{$ELSE}
procedure init(mode:byte);
begin
     asm
     mov ah,0;
     mov al,mode;
     int $10;
     end;
end;
{$ENDIF}

{$IFDEF WIN32}
function farbabodu_pc(x,y:integer):byte;
begin
  farbabodu_pc:=mem.Pixels[x,y]
end;

{$ELSE}

function farbabodu_pc(x,y:integer):byte;
begin
  farbabodu_pc:=mem[320*y+x];
end;
{$ENDIF}

procedure usecka_b(x1, y1, x2, y2: integer; farba: byte);
var i,dx,dy,pixelov,d,dp1,dp2,x,posunx1,posunx2,y,posuny1,posuny2: integer;
begin
  dx:=abs(x2-x1);
  dy:=abs(y2-y1);
  {uhol <= 45}
  if dx>=dy then begin
    pixelov:=dx+1;
    d:=(2*dy)-dx;
    dp1:=dy*2;
    dp2:=(dy-dx)*2;
    posunx1:=1;
    posunx2:=1;
    posuny1:=0;
    posuny2:=1;
  end
  {uhol > 45}
  else begin
    pixelov:=dy+1;
    d:=(2*dx)-dy;
    dp1:=dx*2;
    dp2:=(dx-dy)*2;
    posunx1:=0;
    posunx2:=1;
    posuny1:=1;
    posuny2:=1;
  end;
  {druhy bod je viac vpravo ako prvy bod, otacame suradnice}
  if x1 > x2 then begin
    posunx1:= -posunx1;
    posunx2:= -posunx2;
  end;
  {prvy bod je vyssie ako druhy bod, otacame suradnice}
  if y1 > y2 then begin
    posuny1:= -posuny1;
    posuny2:= -posuny2;
  end;
  {kreslime}
  x:=x1;
  y:=y1;
  for i:=1 to pixelov do begin
    bod_pc(x, y, farba);
    if d < 0 then begin
      d:=d+dp1;
      x:=x+posunx1;
      y:=y+posuny1;
    end
    else begin
      d:=d+dp2;
      x:=x+posunx2;
      y:=y+posuny2;
    end;
  end;
end;

{$IFDEF WIN32}
procedure bod_pc(x,y:integer; farba:byte);
begin
    mem.Pixels[x, y]:=farba;
    mem.Ellipse(x - 2, y - 2, x + 2, y + 2);
end;
{$ELSE}
procedure bod_pc(x,y:integer; farba:byte);
Var Reg: Registers;
begin
  Reg.AH := $0C; {service no}
  Reg.AL := farba; {pt color}
  Reg.BH := $0; {display page no (ignored in 12h}
  Reg.CX := x; {x-coord}
  Reg.DX := y; {y-coord}
  Intr($10, Reg); {call interrupt}
end;
{$ENDIF}

{-----------------------------------------------------------}

procedure PointToPointH(pt:TPoint; var pth:TPointH);
begin
  pth.x:=pt.x;
  pth.y:=pt.y;
  pth.h:=1;
end;

procedure PointHToPoint(pth:TPointH; var pt:TPoint);
  var xx,yy,hh:real;
begin
  xx:=pth.x;
  yy:=pth.y;
  hh:=pth.h;

  pt.x:=xx / hh;
  pt.y:=yy / hh;
end;

procedure PointMatrixTimesMatrix(pt:TPointMatrix; m:TMatrix; var result:TPointMatrix);
  var i,j,k:integer;
begin
  for i:=1 to 3 do begin
    result[i]:=0;
    for j:=1 to 3 do begin
      result[i]:=result[i] + (pt[j]*m[j,i]);
    end;
  end;
end;

procedure PointHToPointMatrix(pth:TPointH; var ptm:TPointMatrix);
begin
  ptm[1]:=pth.x;
  ptm[2]:=pth.y;
  ptm[3]:=pth.h;
end;

procedure PointMatrixToPointH(ptm:TPointMatrix; var pth:TPointH);
begin
  pth.x := ptm[1];
  pth.y := ptm[2];
  pth.h := ptm[3];
end;

procedure multiplyMatrices(m,n: TMatrix; var p: TMatrix);
  var i,j,k:byte;
begin
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
      p[i,j]:=0;
      for k:=1 to 3 do begin
        p[i,j]:=p[i,j]+(m[i,k]*n[k,j]);
     end;
    end;
  end;
end;

procedure getTranslationMatrix(dx, dy:integer; var m:TMatrix);
begin
  m := xlateMatrix;
  m[3,1] := dx;
  m[3,2] := dy;
end;

procedure getRotationMatrix(angle:real; var m:TMatrix);
  var radangle:real;
      cosa, sina:real;
begin
  radangle:=angle * (pi/180);
  cosa:=cos(radangle);
  sina:=sin(radangle);
  m := rotateMatrix;
  m[1,1] := cosa;  m[1,2] := sina;
  m[2,1] := -sina; m[2,2] := cosa;
end;

procedure translatePointH(pt:TPointH; dx,dy:integer; var xpth:TPointH);
  var ptm:TPointMatrix;
      resultPtMatrix:TPointMatrix;
      pth:TPointH;
      xlationMatrix:TMatrix;
begin
  xlationMatrix := xlateMatrix;
  xlationMatrix[3,1] := dx;
  xlationMatrix[3,2] := dy;
  PointHToPointMatrix(pt, ptm);
  PointMatrixTimesMatrix(ptm, xlationmatrix, resultPtMatrix);
  PointMatrixToPointH(resultPtMatrix, xpth);
end;

procedure scalePointH(pt:TPointH; xscale, yscale:real; var xpth:TPointH);
  var ptm:TPointMatrix;
      resultPtMatrix:TPointMatrix;
      pth:TPointH;
      skaleMatrix:TMatrix;
begin
  skaleMatrix := scaleMatrix;
  skaleMatrix[1,1] := xscale;
  skaleMatrix[2,2] := yscale;
  PointHToPointMatrix(pt, ptm);
  PointMatrixTimesMatrix(ptm, skaleMatrix, resultPtMatrix);
  PointMatrixToPointH(resultPtMatrix, xpth);
end;

procedure rotatePointH(pt:TPointH; angle:real; var xpth:TPointH);
  var ptm:TPointMatrix;
      resultPtMatrix:TPointMatrix;
      pth:TPointH;
      rotMatrix:TMatrix;
      cosa:real;
      sina:real;
      radangle:real;
begin
  radangle:=angle * (pi/180);
  cosa:=cos(radangle);
  sina:=sin(radangle);
  rotMatrix := rotateMatrix;
  rotMatrix[1,1] := cosa;  rotMatrix[1,2] := sina;
  rotMatrix[2,1] := -sina; rotMatrix[2,2] := cosa;
  PointHToPointMatrix(pt, ptm);
  PointMatrixTimesMatrix(ptm, rotMatrix, resultPtMatrix);
  PointMatrixToPointH(resultPtMatrix, xpth);
end;

procedure rotatePointHEx(pt:TPointH; angle:real; xoffset, yoffset:integer; var xpth:TPointH);
  var ptm:TPointMatrix;
      resultPtMatrix:TPointMatrix;
      pth:TPointH;
      rotMatrix:TMatrix;
      cosa:real;
      sina:real;
      radangle:real;
      mx1, mx2, mr, mt1: TMatrix;
begin
  getTranslationMatrix(-xoffset, -yoffset, mx1);
  getRotationMatrix(angle, mr);
  getTranslationMatrix(xoffset, yoffset, mx2);

  multiplyMatrices(mx1, mr, mt1);
  multiplyMatrices(mt1, mx2, rotMatrix);

  PointHToPointMatrix(pt, ptm);
  PointMatrixTimesMatrix(ptm, rotMatrix, resultPtMatrix);
  PointMatrixToPointH(resultPtMatrix, xpth);             
end;

end.